% GripTransform.m
% Calculates the grip transform for a stance.
% I have some issues with this, because if you think about the robot 
% "gripping" the ground, especially in the case of a planar ground, you're
% just going to have n points on a plane.
% My preference would be to think about the legs "grasping" the body at the
% hips - this would abstract the friction problem to the middle of the
% kinematic chain, but I think that it would be fair to assume this away at
% first and then come back to verify the original assumption.
%
% Anyway:

clear %for testing
foot_pos=[0 0 0;2 0 0;0 2 0;0 0 2;-2 0 0; 0 -2 0]'; %for testing
robot.leg_angles=[0 0;0 0;0 0;0 0;0 0;0 0]; %for testing

syms phi_s psi_s

% Must run a configurator first, like REAL_RiSE_Comp_Script.m
% really just need robot.leg_angles and foot_pos

feet=[1 3 5];  % Which feet are in contact with the ground

% In the ground frame:
% For point contact with friction:
%  each contact point has three wrenches:
point.w1=[1 0 0 0 0 0]';
point.w2=[0 1 0 0 0 0]';
point.w3=[0 0 1 0 0 0]';
point.w=[point.w1,point.w2,point.w3]; 
% Looks suspiciously like the 'H' matrix...
% for a soft finger, add: w4=[0 0 0 0 0 1];
% 
% By the way, setting up for thinking of the legs grasping the body at the
% hip, you'd have w=[1 0 0 0 0 0
%                    0 1 0 0 0 0
%                    0 0 1 0 0 0
%                    0 0 0 1 0 0
%                    0 0 0 0 1 0];
% Which just leaves rotation about the local z-axis.

%Use velocity transform to convert the wrenches at each foot to the ground
%cooordinate system. This was copied from findCompliance, which deals with
%the foot frame, which is not required for this - only the foot position
%and the contact assumption.

clear gr_Tv;
clear gr_Tf;
for i=1:length(feet)
    g_foot_pos(:,feet(i))=subs(foot_pos(:,feet(i)),{phi_s psi_s},{robot.leg_angles(feet(i),1)*pi/180,robot.leg_angles(feet(i),2)*pi/180});
    rot=eye(3);  %frames are aligned - just doing a force transform along the ground plane
    gPgr = g_foot_pos(1:3,feet(i)); %vector in ground coords from ground origin to the foot
    gPgrX = [0 -gPgr(3) gPgr(2); gPgr(3) 0 -gPgr(1); -gPgr(2) gPgr(1) 0]; %cross product matrix
    gr_Tv(:,:,feet(i)) = [ rot gPgrX*rot; zeros(3) rot];  %Velocity transform from foot to ground origin
  %Theory Testing - just to enforce that gr_Tf = rg_Tv':
    rg_Tv(:,:,feet(i)) = inv(gr_Tv(:,:,feet(i)));
  %End of Theroy Testing
    gr_Tf(:,:,feet(i)) = [ rot zeros(3); gPgrX*rot rot];  %Force transform from foot to ground origin.
    wrench(i).matrix = gr_Tf(:,:,feet(i)) * point.w;      %Transform contact wrench to ground coords
    wrench(i).unisense=[3]; % Column numbers which represent unisense wrenches
end

% Combining to get master W matrix (and using Salisbury's method of keeping
% uni-sense forces at the beginning of the matrix):
clear W;
index=1;
for i=1:length(feet)
    if size(wrench(i).unisense,2) > 0
        for j=1:size(wrench(i).unisense,1)
            W(:,index)=wrench(i).matrix(:,wrench(i).unisense(j));
            index=index+1;
        end
    end
end
% Add in the remaining wrenches
for i=1:length(feet)
    for j=1:size(wrench(i).matrix,2)
        if size(find(wrench(i).unisense==j),2)==0
            W(:,index)=wrench(i).matrix(:,j);
            index=index+1;
        end
    end
end

W

% ch are the basis vectors which flush out W to full rank. - Use SVD to get
% the orthonormal basis for the space NOT spanned by the rows of W.  Gives
% a warning if W is not full rank (i.e., the rows are dependant)
[U,S,V]=svd(W');
flag=0;
if size(find(diag(S)),1) < size(W,1)
    sprintf('Warning!! W matrix is not full rank.\n');
    flag=1;
end
CH=U(:,size(W,1)+1:size(W,2))';

Gtinv = [W;CH]

if flag ~= 1
    Gt = inv(Gtinv)
else
    sprintf('Gtinv not computed\n');
end

%Important Results
%F_res = [fx,fy,fz,mx,my,mz,lam1,lam2, ..., lam(n-6)]' - in this case, this
%is the net force applied TO THE GROUND.
%F_contact = [c1,c2,...,cn]'
%F_contact = Gt*F_res
%F_res = Gtinv*F_contact
%
% V_contact = [d1,d2,...,dn]' (twist intensities per unit time along the
% wrench axes at each contact)
% V_res = [vx,vy,vz,wx,wy,wz,gamma1,gamma2,...,gamma(n-6)]' - body (in this
% case GROUND) velocities and virtual velocities from deformation of the
% GROUND in this case
% V_res = G * V_contact
% V_contact = Ginv * V_res