%function [gC,gCeff] = findComplianceOnly(robot,foot_pos,T_g2_s,H,gwT,wTr,Cjoint,Cfoot,jacobs,feet,options)
%Calculates effective stiffness at the robot CG, calculating effective
% compliance and stiffness at each foot that's in contact with the ground
% along the way - INCLUDES CHANGE WHICH ASSUMES THAT FOOT(ANKLE) COMPLIANCE
% MATRIX IS GIVEN IN A SYSTEM WHOSE Z AXIS IS PARALLEL TO THE GROUND SYSTEM
% Z AXIS - via "options" parameter, utilize different compliances:
%  1=joint only, 2=feet/structural only, 3=total sum
% Stiffness is calculated for options 2 and 3 (not for joint only, which is
% usually singular)

test=0; %Test harness
if test==1
    clear
    clc
    options=3;
    feet=[1 2 3];
    foot_pos=[2 0 0 1;0 3 0 1;-2 0 0 1]';
    robot.leg_posn=zeros(size(foot_pos,2),2);
    robot.leg_angles=zeros(size(foot_pos,2),2);
    robot.leg_axes=[0 1 0;0 1 0;0 1 0];
    robot.cg=[0 0 1];
    for i=1:size(foot_pos,2)
        T_g2_s(:,:,i)=[eye(3) foot_pos(1:3,i);0 0 0 1];
        jacobs = zeros(6,2,size(foot_pos,2));
    end
    gwT=eye(4);
    wTr=eye(4);
    H=[eye(3) zeros(3)];
    Cfoot = diag([0.002 0.002 0.005 50*pi/180 50*pi/180 60*pi/180]);
    Cjoint = diag([1/20 1/20]);
end


syms phi_s psi_s

grT = gwT*wTr;
grR = grT(1:3,1:3);

bKt = zeros(6);
for i=1:length(feet)
   % get velocity transforms from foot to body and tranform the foot compliance
    g_foot_pos(:,feet(i))=subs(foot_pos(:,feet(i)),{phi_s psi_s},{robot.leg_angles(feet(i),1)*pi/180,robot.leg_angles(feet(i),2)*pi/180});
    T_g2(:,:,feet(i)) = subs(T_g2_s(:,:,feet(i)),{phi_s psi_s},{robot.leg_angles(feet(i),1)*pi/180,robot.leg_angles(feet(i),2)*pi/180});
    gPgr = gwT*[robot.cg';1] - g_foot_pos(:,feet(i)); %vector, in ground coords, from foot to robot cg (homogeneous)
    gPgr = gPgr(1:3); %vector, in ground coords, from foot contact point to robot cg 
    gPgrX = [0 -gPgr(3) gPgr(2); gPgr(3) 0 -gPgr(1); -gPgr(2) gPgr(1) 0];
    gr_Tv(:,:,feet(i)) = [ grR gPgrX*grR; zeros(3) grR];
    gfR = T_g2(1:3,1:3,feet(i));
  %modification 'cause RiSE foot is pretty much already in wall coordinates
    coser=dot(gfR(:,3),[0;0;1]);
    siner=norm(cross(gfR(:,3),[0;0;1]));
    Rprime=[coser 0 -robot.leg_axes(feet(i),2)*siner;0 1 0;robot.leg_axes(feet(i),2)*siner 0 coser];
    gfR=Rprime*gfR;
  %end of modification
    gf_Tv(:,:,feet(i)) = [gfR zeros(3);zeros(3) gfR];
    gCfoot = gf_Tv(:,:,feet(i)) * Cfoot * gf_Tv(:,:,feet(i))';
 %Transforms from ankle to ground contact  
    gPgf = robot.ankle_offsets(feet(i),:);  %vector from foot contact to ankle center point (ankle offset) in ground coords
    gPgfX = [0 -gPgf(3) gPgf(2); gPgf(3) 0 -gPgf(1); -gPgf(2) gPgf(1) 0]; %cross product matrix
    gf_Tv(:,:,feet(i)) = [gfR gPgfX*gfR;zeros(3) gfR]; %velocity transform from ankle to ground @ contact point
  %Theory Testing
    gf_Tf(:,:,feet(i)) = [ gfR zeros(3); gPgfX*gfR gfR]; %force transform from ankle to ground in ground coords
    fg_Tv(:,:,feet(i)) = inv(gf_Tv(:,:,feet(i))); %gf_Tf = fg_Tv'
  %End Theory Testing
    gCfoot = gf_Tv(:,:,feet(i)) * Cfoot * gf_Tv(:,:,feet(i))';
  % Tranform joint compliance to the contact coordinates
    gCjoint = jacobs(:,:,feet(i)) * Cjoint * transpose(jacobs(:,:,feet(i)));

    
  % Add serial compliances based on options, 1=joint only, 2=feet/structural only, 3=total sum
    if options==1
        gC(:,:,feet(i))=gCjoint;
    elseif options==2
        gC(:,:,feet(i))=gCfoot;
    else
        gC(:,:,feet(i))=gCfoot+gCjoint;
    end
   % Apply contact constraints
    gCeff(:,:,feet(i)) = H * gC(:,:,feet(i)) * H';
  % Invert to get stiffness, re-apply contact constraints to flesh out 6x6 matrix
    if options > 2
        gK = inv(gCeff(:,:,feet(i)));
        gKeff = H' * gK * H;
        % Transform to CG
        bKeff(:,:,feet(i)) = gr_Tv(:,:,feet(i))' * gKeff * gr_Tv(:,:,feet(i));
        bKt = bKt + bKeff(:,:,feet(i));
    end
end

